# Program under CC license
# Last modified: September 16 2019
# Author: Pierre Bouvet (pierre.bouvet@grenoble-inp.org)

import serial
from tkinter import Toplevel, Spinbox, Text, messagebox
from tkinter.filedialog import asksaveasfile
import Framing_functions as Ff
import Styles as sty
from functools import partial
import sys
import glob
import time
import Pins_Arduino as pa
import Path as pth
import os
import pygame
import serial.tools.list_ports
# from math import *

print_exchange=True
print_command=False
print_function=False
print_all=False

def serial_ports(): #OK
    """ 
    Lists the serial port names accessible at the time of the calling of the function

    Entry: 
        None
    Returns:
        result(list): list of the serial ports available on the system

    :raises EnvironmentError:
        On unsupported or unknown platforms
    """
    ports = list(serial.tools.list_ports.comports())
    for p in ports:
        if "Generic CDC" in p:
            result=str(p[0])
            return result
    messagebox.showerror("Error in connecting to the device", "Please check that the device is connected to the computer")
    return None

def verif_communication(text_sent,text_returned, arduino): #OK
    """
    This function verifies that the arduino has received the right string.
    
    Entry: 
        text_sent(string): the string that is being sent
        text_returned(string): the text that is being returned by the arduino
    Returns:
        Boolean: True if communication OK, False if communication couldn't be established
    """
    if print_function:
        print("Entering verif_communication")
    if (text_returned=="Serial OK"):
        print("Arduino switched off and on again")
        return EnvironmentError
    if text_returned[0:4]=='CMNT':
        print(text_returned)
        text_returned=str(arduino.readline())[2:-5]
        verif_communication(text_sent, text_returned, arduino)
    elif (text_returned!='ACK '+text_sent):
        k=0
        while not (text_returned=='ACK '+text_sent):
            time.sleep(0.01)
            arduino.write(text_sent.encode())
            text_returned=str(arduino.readline())[2:-5]
            k=k+1
            if k>100:
                print("Error in communication: message sent 100 times with a wrong return")
                return EnvironmentError
    return True

def send_and_wait_verif(arduino, text): #OK
    """
    This function sends a string to the arduino and makes sure it receives it correctly
    
    Entry:
        arduino(serial): the serial connection to the Arduino
        text(string): the string that is being transfered to the Arduino
    Returns:
        Boolean: True if communication OK, False if communication couldn't be established
    """
    if print_function:
        print("Entering send_and_wait_verif")
    arduino.write(text.encode())
    if print_exchange:
        print('Data sent    : ' + text)
    received=str(arduino.readline())[2:-5]
    if print_exchange:
        print('Data received: ' + received)
    if verif_communication(text,received,arduino):
        return True
    else:
        return False

def send_value(arduino, text):#OK
    """
    This function sends the information to the arduino, waits for it to be returned by the arduino, compares the two informations, and while the two are not the same, sends them again 

    Entry:
        motor(str): the motor (either the x-axis or the z-axis, respectively named 'X' and 'Z')
        direction(int): the direction the motor has to move(1 for right/up, 2 for left/down)
        step(int): the number of steps
    Returns:
        True at the end of the process
    """
    if print_function:
        print("Entering send_value")
    a=send_and_wait_verif(arduino, text) 
    while not a: 
        a=send_and_wait_verif(arduino, text)
    return True

def init_device(arduino, Window):
    """
    This function reinitializes the device. It begins by activating the magnet. Note that the arduino is activated only when the box is closed.
    It moves the plate and the pipette back to their initial position and reinitializes the pipette
    
    Entry:
        arduino(serial): the connexion between the arduino and the computer

    Returns:
        Boolean: True if no problem, False if there is a problem
    """
    boolean=True
    if print_function:
        print("Entering init_device")
    messagebox.showinfo(title="Beginning protocol", message="Brace yourself, protocol has begun")
    #Here we initialize the pins and the different parameters that the arduino will take into account

    # if print_command:
    #         print("Activating the electromagnet")
    # boolean=set_pin(arduino, 1, pa.Electro_magnet) #Here we activate the electromagnet. Note that the Arduino can only apply commands if the door is closed
    # if print_command:
    #         print("Reinitializing EWOD")
    # boolean=boolean and send_value(arduino, '6')
    # boolean=boolean and send_value(arduino, '403403403403403')
    # if print_command:
    #             print("Reinitializing the pipette")   
    # boolean = boolean and generate_step(arduino, pa.Pip_rei, 1, 100, 1) #Reinitialization of the pipette
    # time.sleep(1) # Waiting for the pipette to reinitialise TODO
    # if print_command:
    #             print("Pressing the TIP button")
    # generate_step(arduino, pa.Pip_tip, 1, 100, 1) # We launch the autoset sequence of the pipette by pressing on the "TIP" button
    # if print_command:
    #             print("Reinitializing the position of the pipette and the plate")
    # send_value(arduino,'7') #Here we reinitialize the position of both the plate and the pipette
    # time.sleep(5) #Here we wait for the pipette to finalise its autosetting sequence
    return boolean

def launch_arduino(Window, values, protocol, Window_main, port='None'): 
    """
    This function gets the parameters of the parameter window of the arduino, and uses them to establish the link between the command board and the PC
    
    Entry:
        Window(frame): the parent frame
        values(spinbox variable): the widget containing the port
        protocol(list): the list containing all the commands
    Returns:
        Nothing BUT activates the function Arduino_command
    """
    if print_function:
        print("Entering launch_arduino")
    if port != 'None':
        port=str(values.get())
    arduino=serial.Serial(port,9600)
    arduino.flush()
    line=str(arduino.readline())[2:-5]
    if not (line=="Serial OK"):
        while not (line=="Serial OK"):
            arduino.close()
            arduino=serial.Serial(port,9600)
            line=str(arduino.readline())[2:-5]
    Window.destroy()
    init_device(arduino, Window_main) #We initialize the device (move the pipette and the plate until they are at their initial position and reinitilize the pipette)
    Arduino_command(Window_main, protocol,arduino)

def launch_arduino_bis(Window_main, protocol, port):
    arduino=serial.Serial(port,9600)
    arduino.flush()
    line=str(arduino.readline())[2:-5]
    if not (line=="Serial OK"):
        while not (line=="Serial OK"):
            arduino.close()
            arduino=serial.Serial(port,9600)
            line=str(arduino.readline())[2:-5]
    init_device(arduino, Window_main) #We initialize the device (move the pipette and the plate until they are at their initial position and reinitilize the pipette)
    Arduino_command(Window_main, protocol,arduino)

def wait_connection(arduino): #OK
    """
    Waits for the arduino to be connected to the computer

    Entry:
        arduino(serial): the serial connection to the arduino
    Returns:
        Boolean: True if communication OK, False if communication couldn't be established

    """
    if print_function:
        print("Entering wait_connection")
    connected=False
    while not connected:
        received=str(arduino.readline())[2:-5]
        connected=True
    if received=='Serial OK':
        return True
    else:
        print("Error in initializing the Arduino")
        return False

def set_pin(arduino, state, pin): #OK 
    """
    This function passes a pin to high state

    Entry:
        arduino(serial): the serial connection to the Arduino
        state(int): the state we want to put the button to (0 for LOW, 1 for HIGH)
        pin(int): the pin we want to activate
    Returns:
        True at the end of the process
    """
    if print_function:
        print("Entering set_pin")
    send_value(arduino,'1')
    send_value(arduino, str(state)+str(100+pin)[1:])
    return True

def generate_step(arduino, pin, steps, half_period, first_state): #OK
    """
    This function generates a step signal

    Entry:
        arduino(serial): the serial connection to the Arduino
        pin(int): the pin we want to activate (must be between 0 and 99)
        steps(int): the number of periods
        half_period(int): half of the period
        first_state(int): the ground state (0 for LOW, 1 for HIGH)
    Returns:
        True at the end of the process
    """
    if print_function:
        print("Entering generate_step")
    send_value(arduino,'2')
    send_value(arduino, str(int(steps))+str(100+pin)[1:]+str(first_state))
    send_value(arduino, str(half_period))
    return True

def move_motor(arduino, actual_position, desired_position, motor): #OK
    """
    This function moves a motor to a desired position (in number of steps)

    Entry: 
        arduino(serial): the serial conection to the Arduino
        actual_position(int): the actual position the motor is in
        desired_positon(int): the targetted position for the motor
        motor(int): the motor to activate, 0 for the X motor, 1 for the Z motor
    Returns:
        Boolean: True if nothing went wrong
    """
    if print_function:
        print("Entering move_motor")
    send_value(arduino, '3')
    if (actual_position<desired_position):
        sens=0
        pas=desired_position-actual_position
    else:
        sens=1
        pas=actual_position-desired_position
    send_value(arduino, str(pas)+str(sens)+str(motor))
    return True

def create_PWM(arduino, pin, value): #OK
    """
    This function genereates a PWM signal

    Entry:
        arduino(serial): the serial connection to the Arduino
        pin(int): the pin we want to activate
        value(int): the value that has to be created by the Arduino (between 0 and 255)
    Returns:
        True at the end of the process
    """
    if print_function:
        print("Entering create_PWM")
    send_value(arduino,'4')
    send_value(arduino, str(value)+str(100+pin)[1:])
    return True

def ask_for_value(arduino, pin, measure, tme): #OK
    """
    This function genereates a PWM signal

    Entry:
        arduino(serial): the serial connection to the Arduino
        pin(int): the pin we want to activate
        measures(int): the number of measures that have to be performed
        tme(int): the time between measures (in ms)
    Returns:
        True at the end of the process
    """
    if print_function:
        print("Entering ask_for_value")
    values_received=[]
    send_value(arduino,'5')
    send_value(arduino, str(measure)+str(100+pin)[1:])
    send_value(arduino,str(tme))
    i=0
    while i<measure:
        received=str(arduino.readline())[2:-5]
        values_received.append(int(received))
        i=i+1
    return values_received

def Arduino_pipet(arduino, volume, solution): #OK
    """
    This function sends the command to the Arduino to pipet the desired amount of solution in the desired solution

    Entry: 
        volume(int): the volume of solution that we want to pipette
        solution(int): the solution in which we want to pipette
        arduino(serial): the serial connection to the Arduino
    Returns:
        Boolean: True if nothing went wrong
    """
    if print_function:
        print("Entering Arduino_pipet")
    motor_steps=pa.motor_steps
    lower=pa.lower_move_pipette#Steps to get from high position to low position

    boolean=True
    change_vol=True
    volume_initial=5000 # TODO nL
    if volume>volume_initial: #We have to ask the pipette to pipette more than 5µL
        pin1=pa.Pip_upp #This is the pin that corresponds to the more on the pipette
        pin2=pa.Pip_dow #This is the pin that corresponds to the less on the pipette
        steps=(volume-volume_initial)/50
    elif (volume<volume_initial): #We have to ask the pipette to pipette less than 5µL
        pin1=pa.Pip_dow #This is the pin that corresponds to the less on the pipette
        pin2=pa.Pip_upp #This is the pin that corresponds to the more on the pipette
        steps=(volume_initial-volume)/50
    else: #We don't have to change the parameters of the pipette
        change_vol=False

    steps_tip=motor_steps[solution-3]#We first have to take the tip so we look how many steps there are from the reference position to the tip
    steps_sol=motor_steps[solution-1]-steps_tip

    while boolean:
        if change_vol: #If we have to change the volume the pipette has to pipette
            if print_all:
                print("Changing the volume on the pipette")
            boolean=boolean and generate_step(arduino, pin1, steps, pa.tmp_btn, 1)# Press a certain number of times on the + button
            boolean=boolean and generate_step(arduino,pa.Pip_ent,1,pa.tmp_btn,1)# Press one time on the enter button
        
        #Move the plate to the tips
        if print_all:
            print("Moving the plate")
        boolean=boolean and move_motor(arduino, 0, steps_tip, 0)
        
        #Move the pipette to take a tip
        if print_all:
            print("Lower the pipette to take a tip")
        boolean=boolean and move_motor(arduino, 0, lower, 1)
        if print_all:
            print("Raise the pipette with the tip")
        boolean=boolean and move_motor(arduino, lower, 0, 1)

        #Move the plate to the solution
        if print_all:
            print("Move the plate")
        boolean=boolean and move_motor(arduino, 0, steps_sol, 0)

         #Move the pipette and pipette sample
        if print_all:
            print("Lower the pipette to take a sample")
        boolean=boolean and move_motor(arduino, 0, lower, 1)
        if print_all:
            print("Pipette a sample")
        boolean=boolean and generate_step(arduino, pa.Pip_sta, 1, 20,1)# Pipette sample
        if print_all:
            print("Raise the pipette holding the sample")
        boolean=boolean and move_motor(arduino, lower, 0, 1)

        #Move the plate to the EWOD plate
        if print_all:
            print("Move the plate")
        boolean=boolean and move_motor(arduino, steps_sol+steps_tip, 0, 0)

        #Move the pipette and release sample
        if print_all:
            print("Lower the pipette to release the sample")
        boolean=boolean and move_motor(arduino, 0, lower, 1)
        if print_all:
            print("Release the sample")
        boolean=boolean and generate_step(arduino, pa.Pip_sta, 1, 20,1)# Pipette sample
        if print_all:
            print("Raise the pipette holding the sample")
        boolean=boolean and move_motor(arduino, lower, 0, 1)

        #Move the plate to the tips
        if print_all:
            print ("Moving the plate")
        boolean=boolean and move_motor(arduino, 0 , steps_tip, 0)

        #Move the pipette and release tip
        if print_all:
            print("Lower the pipette to eject the tip")
        boolean=boolean and move_motor(arduino, 0, lower, 1)
        if print_all:
            print("Eject the tip")
        boolean=boolean and generate_step(arduino, pa.Pip_tip, 1, 20,1)# Eject tip
        if print_all:
            print("Raise the pipette")
        boolean=boolean and move_motor(arduino, lower, 0, 1)
         
        #Move the plate to the tips
        if print_all:
            print("Coming back to initial position")
        boolean=boolean and move_motor(arduino, steps_tip , 0, 0)

        if change_vol: #If we have to change the volume the pipette has to pipette
            if print_all:
                print("Changing the volume on the pipette")
            boolean=boolean and generate_step(arduino, pin2, steps, 10, 1)

        return boolean
    return boolean

def convert_to_send(pad):
    """
    This is a function that converts a pad in the form [a.b] onto a string to send to the HV chips to activate the pad

    Entry: 
        pad: The pad to activate of the form [a,b]
    Returns 
        send: The string to send
    """
    send=''
    temp=((pad[1]-1)%8)+1
    send=send+str(temp)
    temp=7-(pad[1]-1)//8+8*(pad[0]-1)
    temp='00'+str(temp)
    send=send+temp[-2:]
    return send

def Arduino_move(xbeg,ybeg,xend,yend,arduino, Pads_occupied): #OK
    """
    This function sends the command to the Arduino to move the solution on the EWOD plate

    Entry: 
        arduino(serial): the serial connection to the Arduino
        xbeg(int): x coordinate of the beginning position
        ybeg(int): y coordinate of the beginning position
        xend(int): x coordinate of the ending position
        yend(int): y coordinate of the ending position
        Pads_occupied(list): the list of pads that are currently occupied

    Returns:
        Boolean: True if nothing went wrong
    """    
    if print_function:
        print("Entering Arduino_move")
    boolean=True
    path=pth.create_path(xbeg, ybeg, xend, yend, Pads_occupied)
    fichier=open('list_pad.csv','r')
    values= fichier.readlines()
    fichier.close()
    for i in range(len(values)):
        values[i]=values[i].replace('\n','')
        values[i]=values[i].split(';')
        for k in range(len(values[i])):
            if (values[i][k]=='0'):
                values[i][k]=None
            else:
                values[i][k]=values[i][k].split('.')
                values[i][k][0]=int(values[i][k][0])
                values[i][k][1]=int(values[i][k][1])
    pads_to_activate=[]
    for step in path:
        if ((values[step[2]]==None) or (values[step[3]]==None)):
            print("Problem comes from Arduino_move: Wrong pad coordinate")
            return ValueError
        else:
            pads_to_activate.append(values[step[2]][step[3]])
    for pad in pads_to_activate:
        if print_all:
            print(pad)
        send=convert_to_send(pad)
        send_value(arduino, '6')
        send_value(arduino, send)
        time.sleep(1)
    return boolean

# arduino=serial.Serial('/dev/cu.usbmodem14101', 9600)
# arduino.flush()
# lect=str(arduino.readline())[2:-5]
# print(lect)
# # Arduino_move(30,42,30,22,arduino, [])
# # Arduino_move(30,22,30,20,arduino, [])

# pad=[0,1]
# while True:
#     pad[0]=int(input("Chip Number: "))
#     pad[1]=int(input("Pad Number : "))
#     print(pad)
#     send=convert_to_send(pad)
#     send_value(arduino, send)

# # for k in range(1,6):
# #     i=64
# #     while i>0:
# #         print([k,i])
# #         send=convert_to_send([k,i])
# #         send_value(arduino, send)
# #         i=i-1



def Arduino_measure(nbmeasures, t_between_measures, pin,  arduino): #OK
    """
    This function sends the command to the Arduino to pipet the desired amount of solution in the desired solution

    Entry: 
        arduino(serial): the serial connection to the Arduino
        nbmeasures(int): number of measurements to perform
        t_between_measures(int): time between two measurements
    Returns:
        Boolean: True if nothing went wrong
    """
    if print_function:
        print("Entering Arduino_measure")
    values=ask_for_value(arduino,pin,nbmeasures,t_between_measures)
    for i in range(len(values)):
        values[i]=str(values[i])+'\n'
    fichier=open('Measures_performed.txt','w')
    fichier.writelines(values)
    fichier.close()
    return True

def Arduino_clean(arduino): #TODO
    """
    This function sends the command to the Arduino to pipet the desired amount of solution in the desired solution

    Entry: 
        volume(int): the volume of solution that we want to pipette
        solution(int): the solution in which we want to pipette
        arduino(serial): the serial connection to the Arduino
    Returns:
        Boolean: True if nothing went wrong
    """
    boolean=True
    if print_function:
        print("Entering Arduino_clean")
    boolean=send_value(arduino,'Cle')
    file = 'Dracarys.mp3'
    pygame.mixer.init()
    pygame.mixer.music.load(file)
    pygame.mixer.music.play()
    time.sleep(8)
    if not boolean:
        print("Problem comes from Arduino_clean")
    return boolean

def Arduino_thaw(arduino): #OK TODO: fix the value to wait for thaw
    """
    This function sends the command to the Arduino to pipet the desired amount of solution in the desired solution

    Entry: 
        volume(int): the volume of solution that we want to pipette
        solution(int): the solution in which we want to pipette
        arduino(serial): the serial connection to the Arduino
    Returns:
        Boolean: True if nothing went wrong
    """  
    if print_function:
        print("Entering Arduino_thaw")  
    time.sleep(1) 
    return True

def Arduino_temperature(arduino, zone, temp): #TODO
    """
    This function imposes a certain temperature to a zone

    Entry:
        arduino(serial): the serial connection to the Arduino
        zone(int): the zone that we want to control
        temp(float): the temperature we want to impose

    Returns:
        True at the end of the process
    """
    if print_function:
        print("Entering Arduino_temperature")
    send_value(arduino,'9')
    tempo=str(100*temp)+'000'
    if temp<10:
        tempo='0'+tempo
    tempo=tempo[0:4]
    send_value(arduino, tempo+str(zone))
    return True

def Apply_protocol(Window, protocol): #OK
    """
    This function initializes the communication between the Arduino and the computer

    Entry:
        Window(Frame): the parent frame
    Returns:
        parameters(list): list of parameters (serial port)
    """
    if print_function:
        print("Entering Apply_protocol")
    # #We start by creating the parameter window, where we will select the serial port and we will create the directory where all the datas will be stored
    # Parameter_window=Toplevel(Window)
    # Parameter_window.title('Neurodrop - Arduino parameters')
    # Heading=Ff.Create_Heading(Parameter_window)
    # #We recover all the ports present on the computer
    serial_ports_available=serial_ports()
    launch_arduino_bis(Window,protocol, serial_ports_available)
    # #We create the frames
    # Arduino_parameters=Ff.Create_Frame(Parameter_window, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=3, nbcolumns=8, form='wens')
    # Confirm_parameters=Ff.Create_Frame(Parameter_window, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=6, nblines=3, nbcolumns=8, form='wens')
    # # We create the labels where the arduino port can be selected
    # Ff.Create_text(Arduino_parameters, 'Selection of the parameters of the Arduino', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    # Spin=Ff.Create_spinbox(Arduino_parameters, values=serial_ports_available, width=20, begin=0, end=0, step=1, col=1, line=3, form='wens', nbcolumns=1, nblines=1)
    # Ff.Create_text(Confirm_parameters, 'Save parameters', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    # Ff.Create_button(Confirm_parameters, 'Confirm & Apply parameters', partial(launch_arduino, Parameter_window, Spin, protocol, Window), color=sty.Clr_but_whit, activefg=sty.Bkg_button_active, Width=30, Height=2, Police='Arial 25', col=1, line=3, form='wens', nblines=1)
    # #Configure the frames
    # Ff.Auto_configure(Parameter_window, lines=[0,2,3,4,6,7,8], columns=[1,2,3,4,5,6,7,8], min_size_col=10, min_size_row=10,weight_col=1,weight_row=1)
    # Ff.Auto_configure(Parameter_window, lines=[1,5,9], columns=[0,9], min_size_col=10, min_size_row=10,weight_col=0,weight_row=0)    
    # Ff.Auto_configure(Heading, lines=[0], columns=[0,1,2,3,4,5,6,7], min_size_row=10, min_size_col=30, weight_col=1, weight_row=0)
    # Ff.Auto_configure(Arduino_parameters, lines=[0,2,4], columns=[0,2], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
    # Ff.Auto_configure(Arduino_parameters, lines=[1,3], columns=[1], min_size_row=10, min_size_col=10, weight_col=0, weight_row=0)
    # Ff.Auto_configure(Confirm_parameters, lines=[0,2,4], columns=[0,2], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
    # Ff.Auto_configure(Confirm_parameters, lines=[1,3], columns=[1], min_size_row=10, min_size_col=10, weight_col=0, weight_row=0)
    return Window

def end_protocol(arduino):
    """
    This function ends the protocol: it moves everything back to its initial position, and deactivates the magnet

    Entry:
        arduino(serial): the serial connection between the arduino and the computer
    
    Returns: 
        True when it's done
    """
    if print_function:
        print("Entering end_protocol")
    if print_command:
                print("Reinitializing the position of the pipette and the plate")
    send_value(arduino,'7') #Here we reinitialize the position of both the plate and the pipette
    send_value(arduino,'8') #Now we put every pin to LOW value
    return True

def save_values(Window):
    """
    This function opens a tab to save the values that have been taken during the experiment
    It only saves the values read by the SiPM
    """
    if print_function:
        print("Entering save_values")
    fichier=open('Measures_performed.txt','r')
    values=fichier.readlines()
    fichier.close()
    os.remove('Measures_performed.txt')
    filename=asksaveasfile(title="Save the data as",filetypes=[('txt files','.txt'),('all files','.*')])
    filename=str(filename)
    filename=filename.split("'")
    filename=filename[1]
    fichier=open(filename,'w')
    fichier.writelines(values)
    fichier.close()
    return True

def Arduino_command(Window_main, protocol, arduino): #OK
    """
    This function commands the Arduino so it applies the protocol that has previously been selected

    Entry: 
        protocol(list): the protocol
        arduino(serial): the serial communication between the arduino and the computer
    """
    if print_function:
        print("Entering Arduino_command")
    Measures_were_made=False
    Pads_occupied=[]
    for step in protocol:
        cmd=step[0]
        if cmd=='pip':
            vol=step[1]
            sol=step[2]
            if print_command:
                print("Pipette "+str(vol)+"µL in solution "+str(sol))
            Arduino_pipet(arduino, vol, sol) 
        elif cmd=='mov':
            xbeg=step[1]
            ybeg=step[2]
            xend=step[3]
            yend=step[4]
            if print_command:
                print("Move from ("+str(xbeg)+","+str(ybeg)+") to ("+str(xend)+","+str(yend)+")")
            Arduino_move(xbeg, ybeg, xend, yend, arduino, Pads_occupied)
            if ((xbeg,ybeg) in Pads_occupied):
                Pads_occupied.remove((xbeg, ybeg))
            Pads_occupied.append((xend,yend))
        elif cmd=='lum':
            nbmes=step[1]
            t_inter_mes=step[2]
            if print_command:
                print("Do "+str(nbmes)+" measures waiting "+str(t_inter_mes)+"ms between each measure")
            Arduino_measure(nbmes,t_inter_mes,0,arduino)
            Measures_were_made=True
        elif cmd=='cle':
            if print_command:
                print("Clean the device")
            Arduino_clean(arduino)
        elif cmd=='tha':
            if print_command:
                print("Thaw the solution")
            Arduino_thaw(arduino)
        elif cmd=='tem':
            zone=step[1]
            temp=step[2]
            if print_command:
                print("Fix zone "+str(zone)+"to "+str(temp)+"°C")
            Arduino_temperature(arduino,zone,temp)
        elif cmd=='wai':
            tim=step[1]
            if print_command:
                print("Wait "+str(tim)+"seconds")
            time.sleep(tim)
        elif cmd!='position':
            print("THERE IS AN ERROR IN THE COMMANDS")
            return ValueError
    end_protocol(arduino)
    if Measures_were_made:
        save_values(Window_main)
    arduino.close()
    print("SNAFU")
    

