# Program under CC license
# Last modified: September 16 2019
# Author: Pierre Bouvet (pierre.bouvet@grenoble-inp.org)

from tkinter import Tk, Canvas, PhotoImage, Text, Menu, Button, Frame, Label
from functools import partial
import Framing_functions as Ff
import Styles as sty
import Protocol as ffp
import os

# =============================================================================
# Create the main window using the Tkinter library
# =============================================================================

Window_main=Tk() 
Window_main.title('Neurodrop - Main window')# Setting the title of the window

# =============================================================================
# Organize the main window and add titles
# =============================================================================

#Parts of the main window
Heading=Frame(Window_main)
Heading.grid(column=0, row=0, rowspan=1, columnspan=9, sticky='wens')
Protocol=Frame(Window_main, bd=5, relief='groove',background=sty.Bkg_frm_blue)
Protocol.grid(column=1, row=2, rowspan=3, columnspan=3, sticky='wens')
Measure=Frame(Window_main, bd=5, relief='groove',background=sty.Bkg_frm_blue)
Measure.grid(column=1, row=6, rowspan=3, columnspan=3, sticky='wens')
Logofig=Frame(Window_main, bd=0, relief='flat',background=sty.Bkg_frm_white)
Logofig.grid(column=6, row=2, rowspan=3, columnspan=3, sticky='wens')
Current=Frame(Window_main, bd=5, relief='groove',background=sty.Bkg_frm_blue)
Current.grid(column=6, row=6, rowspan=3, columnspan=3, sticky='wens')

#Heading
Label(Heading,text='Neurodrop',foreground=sty.Clr_txt_Neuro,font=sty.Font_Neur,background=sty.Bkg_txt_Neuro).grid(column=0,row=0, columnspan=4, rowspan=1, sticky='wens')
Label(Heading,text='iGEM Grenoble 2019',foreground=sty.Clr_txt_iGEM,font=sty.Font_iGEM,background=sty.Bkg_txt_iGEM).grid(column=4,row=0, columnspan=4, rowspan=1, sticky='wens')
Label(Heading,text=sty.nb_version,foreground=sty.Clr_txt_vers,font=sty.Font_vers,background=sty.Bkg_txt_vers).grid(column=8,row=0, columnspan=2, rowspan=1, sticky='wens')

#Titles of the parts
Label(Protocol,text='Measurement Protocol',foreground=sty.Clr_txt_frames,font=sty.Font_frames_title,background=sty.Bkg_frm_blue).grid(column=1,row=1, columnspan=1, rowspan=1, sticky='wens')
Label(Measure,text='Apply Protocol',foreground=sty.Clr_txt_frames,font=sty.Font_frames_title,background=sty.Bkg_frm_blue).grid(column=1,row=1, columnspan=1, rowspan=1, sticky='wens')
Label(Current,text='Path of the protocol selected',foreground=sty.Clr_txt_frames,font=sty.Font_frames_title,background=sty.Bkg_frm_blue).grid(column=1,row=1, columnspan=1, rowspan=1, sticky='wens')

# =============================================================================
# Fills the parts of the main window
# =============================================================================

#Protocol Selected
ProtocolSelected=Label(Current,text='None Selected',foreground=sty.Clr_txt_frames,font=sty.Font_button,background=sty.Bkg_frm_blue)
ProtocolSelected.grid(column=1,row=3, columnspan=1, rowspan=1, sticky='wens')
ProtocolLine=Text(Current, fg=sty.Clr_txt_frames,font=sty.Font_filepath,bg=sty.Bkg_frm_blue, width=50, height=4)
ProtocolLine.grid(column=1,row=5, columnspan=1, rowspan=1, sticky='wens')

#Protocols
Button(Protocol, text='Create a new Protocol', foreground=sty.Clr_but_whit, activeforeground=sty.Bkg_button_active, width=30, height=2, font='Arial 25', command=partial(ffp.CreateNewProtocol, Window_main), pady=0.1).grid(column=1,row=3,sticky='wens', rowspan=1)
Button(Protocol, text='Modify an existing Protocol', foreground=sty.Clr_but_whit, activeforeground=sty.Bkg_button_active, width=30, height=2, font='Arial 25', command=partial(ffp.ModifyProtocol, Window_main), pady=0.1).grid(column=1,row=5,sticky='wens', rowspan=1)

#Measurements
Button(Measure, text='Load a protocol', foreground=sty.Clr_but_whit, activeforeground=sty.Bkg_button_active, width=30, height=2, font='Arial 25', command=partial(ffp.OpenMeasureProtocol, Window_main, ProtocolSelected, ProtocolLine), pady=0.1).grid(column=1,row=3,sticky='wens', rowspan=1)
Button(Measure, text='Begin Measure', foreground=sty.Clr_but_begin, activeforeground=sty.Bkg_button_active, width=30, height=2, font='Arial 25', command=partial(ffp.BeginMeasurement, Window_main, ProtocolLine), pady=0.1).grid(column=1,row=5,sticky='wens', rowspan=1)


# Logo
image=PhotoImage(file=os.getcwd()+"/Logo.png") # We select the image file
height=image.height()+10 # We will define the window in function of the size of the logo we have selected
width=image.width()+10
Image_neurodrop=Canvas(Logofig,width=width,height=height)
Image_neurodrop.create_image(int(width/2),int(height/2),image=image)
Image_neurodrop.grid(column=2,row=1,sticky='wens')

# =============================================================================
# Menu for the main window
# =============================================================================



# =============================================================================
# Positionning the different elements
# =============================================================================

Ff.Auto_configure(Window_main, lines=[1,5,9], columns=[0,4,5,9], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
Ff.Auto_configure(Window_main, lines=[0,2,3,4,6,7,8], columns=[1,2,3], min_size_row=10, min_size_col=10, weight_col=1, weight_row=0)
Ff.Auto_configure(Heading, lines=[0], columns=[0,1,2,3,4,5,6,7], min_size_row=10, min_size_col=30, weight_col=1, weight_row=0)
Ff.Auto_configure(Protocol, lines=[0,2,6], columns=[0,2], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
Ff.Auto_configure(Protocol, lines=[1,3,4,5], columns=[1], min_size_row=10, min_size_col=10, weight_col=0, weight_row=0)
Ff.Auto_configure(Measure, lines=[0,2,6], columns=[0,2], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
Ff.Auto_configure(Measure, lines=[1,3,4,5], columns=[1], min_size_row=10, min_size_col=10, weight_col=0, weight_row=0)
Ff.Auto_configure(Logofig, lines=[0,2], columns=[0,2], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
Ff.Auto_configure(Logofig, lines=[1], columns=[1], min_size_row=10, min_size_col=10, weight_col=0, weight_row=0)
Ff.Auto_configure(Current, lines=[0,2,4], columns=[0,2], min_size_row=10, min_size_col=10, weight_col=1, weight_row=1)
Ff.Auto_configure(Current, lines=[1,3], columns=[1], min_size_row=10, min_size_col=10, weight_col=0, weight_row=0)

Window_main.mainloop()