# Program under CC license
# Last modified: September 16 2019
# Author: Pierre Bouvet (pierre.bouvet@grenoble-inp.org)

def create_list_pads(name='list_pad.csv'): #OK
    """
    This function takes in the file where the architecture of the EWOD plate is defined and returns a list of lists of booleans defining if there is a pad or not at the position

    Entry:
        Nothing (the name of the file with the architecture of the pads is predefined)

    Returns:
        List(list): the list of list of booleans that says if pad [i,j] is a pad or not
    """
    fichier=open(name,'r')
    List=fichier.readlines()
    fichier.close()
    for i in range(len(List)):
        List[i]=List[i].replace('\n',"")
        List[i]=List[i].split(';')
    List[0][0]=List[0][0][1:]
    for l in range(len(List)):
        for col in range(len(List[l])):
            if((List[l][col]=='0')or((List[l][col]==''))):
                List[l][col]=False
            else:
                List[l][col]=True
    return List

def transform_pad_accessible(List_pads, lin, col): #OK
    """
    This function returns the list of the pads with each pad being accessible through (lin,col) equal to '1'

    Entry: 
        List_pads(list): list of tuples designing the pads that are currently occupied
        lin(int): the line where we want to put the drop
        col(int): the column where we want to put the drop
    Returns:
        List_pads modified (with each pad accessible directly by (lin,col) raised to the value '1')
    """
    max_lin=len(List_pads)-1
    max_col=len(List_pads[0])-1
    List_pads[lin][col]='1'
    if ((col>0)):
        if (List_pads[lin][col-1]==True):
            List_pads[lin][col-1]='1'
            List_pads=transform_pad_accessible(List_pads,lin,col-1)
    if ((col<max_col)):
        if (List_pads[lin][col+1]==True):
            List_pads[lin][col+1]='1'
            List_pads=transform_pad_accessible(List_pads,lin,col+1)
    if ((lin>0)):
        if (List_pads[lin-1][col]==True):
            List_pads[lin-1][col]='1'
            List_pads=transform_pad_accessible(List_pads,lin-1,col)
    if ((lin<max_lin)):
        if (List_pads[lin+1][col]==True):
            List_pads[lin+1][col]='1'
            List_pads=transform_pad_accessible(List_pads,lin+1,col)
    return List_pads

def put_to_false_square(List_pads, lin,col): #OK
    """
    This function puts the pads around a designed pad to False

    Entry:
        List_pads(list): the list of list
        lin(int): the line
        col(int): the column
    """
    max_lin=len(List_pads)
    max_col=len(List_pads[0])
    List_pads[lin][col]=False
    if col-1>=0:
        List_pads[lin][col-1]=False
    if col+1<=max_col:
        List_pads[lin][col+1]=False
    if lin-1>=0:
        List_pads[lin-1][col]=False
        if col-1>=0:
            List_pads[lin-1][col-1]=False
        if col+1<=max_col:
            List_pads[lin-1][col+1]=False
    if lin+1<=max_lin:
        List_pads[lin+1][col]=False
        if col-1>=0:
            List_pads[lin+1][col-1]=False
        if col+1<=max_col:
            List_pads[lin+1][col+1]=False
    return List_pads

def passage(List_pads, lin, col, xbeg=False, ybeg=False, xend=False, yend=False): #OK
    """
    This function returns True if by placing a drop on (lin,col) we can still access each other electrodes on the plate from any electrode. If not it returns False
    This function is meant to be used with the existing drops on the EWOD plate

    Entry:
        List_pads(list): list of tuples designing the pads that are currently occupied
        lin(int): the line where we want to put the drop
        col(int): the column where we want to put the drop
    
    Returns:
        boolean: True if we can still access any electrode, False if not
    """
    Pads=List_pads.copy()
    for i in range(len(Pads)):
        Pads[i]=List_pads[i].copy()
    Pads=put_to_false_square(Pads, lin, col)
    max_col=len(Pads[0])-1

    
    if (type(xbeg) == int):    
        [x0,y0]=[xbeg,ybeg]
        temp=transform_pad_accessible(Pads,x0,y0) #Now  each pad that can be accessed is at the value '1' 
        if ((temp[xbeg][ybeg]=='1') & (temp[xend][yend]=='1')):
            return True
        else:
            return False
    else:
        num_pads=0 #Here we are counting the total number of pads
        for i in range(len(Pads)):
            for j in range(len(Pads[0])):
                if Pads[i][j]:
                    num_pads=num_pads+1
        [x0,y0]=[0,0]
        while not Pads[x0][y0]:#Now we look for the first pad starting from the top of the list
            if y0<max_col:
                y0=y0+1
            else:
                x0=x0+1
                y0=0
        temp=transform_pad_accessible(Pads,x0,y0) #Now  each pad that can be accessed is at the value '1' 
        for i in range(len(temp)):#Here we are counting the total number of pads
            for j in range(len(temp[0])):
                if temp[i][j]=='1':
                    num_pads=num_pads-1
                    Pads[i][j]=True
        if num_pads>0:
            return False
        elif num_pads==0:
            return True
        else:
            print('Error comes from passage')
            return ValueError

def move_to_adapt(List_pads, lin, col, xbeg, ybeg, xend, yend):
    """
    This function moves a drop so that the path is accessible

    Entry: 
        List_pads(list): list of tuples designing the pads that are currently occupied
        lin(int): the line where we want to put the drop
        col(int): the column where we want to put the drop
    
    Returns:
        path_to_adapt(list): the path to adapt the path
    """
    path_to_adapt=[]
    max_lin=len(List_pads)-1
    max_col=len(List_pads[0])-1

    if ((col>0) & (List_pads[lin][col-1]) & (passage(List_pads, lin, col-1, xbeg, ybeg, xend, yend))): #We try to move to the precedent column
        path_to_adapt.append([lin,col,lin,col-1])
        put_to_false_square(List_pads, lin, col-1)
        return path_to_adapt
    elif((col<max_col) & (List_pads[lin][col+1]) & (passage(List_pads, lin, col+1, xbeg, ybeg, xend, yend))): #We try to move to the following column
        path_to_adapt.append([lin,col,lin,col+1])
        put_to_false_square(List_pads, lin, col+1)
        return path_to_adapt
    elif ((lin>0) & (List_pads[lin-1][col]) & (passage(List_pads, lin-1, col, xbeg, ybeg, xend, yend))): #We try to move to the precedent line
        path_to_adapt.append([lin,col,lin-1,col])
        put_to_false_square(List_pads, lin-1, col)
        return path_to_adapt
    elif((lin<max_lin) & (List_pads[lin+1][col]) & (passage(List_pads, lin+1, col, xbeg, ybeg, xend, yend))): #We try to move to the following line
        path_to_adapt.append([lin,col,lin+1,col])
        put_to_false_square(List_pads, lin+1, col)
        return path_to_adapt
    else:
        print("Can't find a path by moving the drop only on one drop")
        return ValueError

def shortest_path_increment(List_pads, lin, col): #OK
    """
    This function takes a pad and compares its value to its neighbours. If its value is lower than the one of their neighbours minus 1, then we change their value for the one of the one of the pad plus one
    
    Entry:
        List_pads(list): the list of the pads on the EWOD plate
        lin(int): the line to observe
        col(int): the column to observe
    
    Return:
        List_pads(list): the pad list with each pad having the length of the shortest path leading to them
    """
    max_lin=len(List_pads)-1
    max_col=len(List_pads[0])-1
    bool=False
    
    if ((lin>0)):
        if (List_pads[lin-1][col]>List_pads[lin][col]+1):
            List_pads[lin-1][col]=List_pads[lin][col]+1
            List_pads=shortest_path_increment(List_pads, lin-1, col)
            bool = bool or True
    if ((lin<max_lin)):
        if (List_pads[lin+1][col]>List_pads[lin][col]+1):
            List_pads[lin+1][col]=List_pads[lin][col]+1
            List_pads=shortest_path_increment(List_pads, lin+1, col)
            bool = bool or True
    if ((col>0)):
        if(List_pads[lin][col-1]>List_pads[lin][col]+1):
            List_pads[lin][col-1]=List_pads[lin][col]+1
            List_pads=shortest_path_increment(List_pads, lin, col-1)
            bool = bool or True
    if ((col<max_col)):
        if (List_pads[lin][col+1]>List_pads[lin][col]+1):
            List_pads[lin][col+1]=List_pads[lin][col]+1
            List_pads=shortest_path_increment(List_pads, lin, col+1)
            bool = bool or True
    return List_pads

def shortest_path(List_pads, xbeg, ybeg, xend, yend): #OK
    """
    This functions computes the shortest path between two pads

    Entry: 
        List_pads(list): the list of the pads on the EWOD plate
        xbeg(int): the x coordinate of beginning
        ybeg(int): the y coordinate of beginning
        xend(int): the x coordinate of ending
        yend(int): the y coordinate of ending
    
    Returns:
        path(list): the path to follow
    """
    L=List_pads.copy()
    max_lin=len(List_pads)-1
    max_col=len(List_pads[0])-1
    for i in range(len(List_pads)):#Here we initialize the pads to perform the Djikstra algorithm
        for j in range(len(List_pads[i])):
            if List_pads[i][j]:
                L[i][j]=1000
            else:
                L[i][j]=0
    L[xbeg][ybeg]=1
    L=shortest_path_increment(List_pads, xbeg, ybeg)
    pos=[xend,yend]
    path=[]
    while pos!=[xbeg,ybeg]:
        if ((pos[0]>0) & (L[pos[0]][pos[1]]==L[pos[0]-1][pos[1]]+1)):
            path=[[pos[0]-1,pos[1],pos[0],pos[1]]]+path
            pos=[pos[0]-1, pos[1]]
        elif ((pos[1]>0) & (L[pos[0]][pos[1]]==L[pos[0]][pos[1]-1]+1)):
            path=[[pos[0],pos[1]-1,pos[0],pos[1]]]+path
            pos=[pos[0], pos[1]-1]
        elif ((pos[0]<max_lin) & (L[pos[0]][pos[1]]==L[pos[0]+1][pos[1]]+1)):
            path=[[pos[0]+1,pos[1],pos[0],pos[1]]]+path
            pos=[pos[0]+1, pos[1]]
        elif ((pos[1]<max_col) & (L[pos[0]][pos[1]]==L[pos[0]][pos[1]+1]+1)):
            path=[[pos[0],pos[1]+1,pos[0],pos[1]]]+path
            pos=[pos[0], pos[1]+1]
        else:
            print(pos)
            fichier=open("ERROR.csv",'w') #If there has been an error, we write the path onto a csv file.
            K=[]
            for i in range(len(L)):
                K.append('')
                for j in range(len(L[i])):
                    K[i]=K[i]+str(L[i][j])+','
                K[i]=K[i][:-1]+'\n'
            fichier.writelines(K)
            fichier.close()
            print("Error comes from shortest_path")
            return ValueError
    return path

def create_path(xbeg, ybeg, xend, yend, Pads_occupied=[]): #OK
    """
    This function create the path that has to be activated for a drop to move from (xbeg,ybeg) to (xend,yend)

    Entry:
        xbeg(int): the x coordinate of beginning
        ybeg(int): the y coordinate of beginning
        xend(int): the x coordinate of ending
        yend(int): the y coordinate of ending
        Pads_occupied(list): list of tuples designing the pads that are currently occupied
        track(list): the list of precedent steps
    
    Returns: 
        track(list): the list of the steps
    """
    #The idea is to work on the pads that we can navigate on, displacing the drops momentarily if needed
    List_pads=create_list_pads()
    track=[]
    adapt=[]
    if ((Pads_occupied!=None) and ((xend,yend) in Pads_occupied)): #This means that we want to mix to drops. Therefore we need to be able to move to that drop
        Pads_occupied=Pads_occupied.remove((xend,yend))
    if ((Pads_occupied!=None) and ((xbeg,ybeg) in Pads_occupied)):
        Pads_occupied=Pads_occupied.remove((xbeg,ybeg))
    if Pads_occupied!=None:
        for oc in Pads_occupied:
            if passage(List_pads, oc[0], oc[1]): #In this case the drop doesn't bloc the access to the pads
                List_pads=put_to_false_square(List_pads, oc[0], oc[1])#We therefore redefine List_pads with the pads that are now not accessible
            else: #In this case the drop blocs the access to some pads, and so we have to move the blocing drop.
                adapt=adapt+(move_to_adapt(List_pads, oc[0], oc[1], xbeg, ybeg, xend, yend))
        track=track+adapt.copy()#We move the drops that block the movement
    temp=shortest_path(List_pads, xbeg, ybeg, xend, yend)
    track=track+temp
    for i in range(len(adapt)):
        a=adapt[i][:2]
        b=adapt[i][2:]
        adapt[i]=b+a
    track=track+adapt#Now we move back the drops where they were.
    return track
 