# Program under CC license
# Last modified: September 16 2019
# Author: Pierre Bouvet (pierre.bouvet@grenoble-inp.org)

import Protocol as Prot
import os
from tkinter import Spinbox, Toplevel, IntVar, Entry, Listbox, Scale, Radiobutton, StringVar, Canvas, Scrollbar, messagebox, Button, Label
from tkinter.filedialog import asksaveasfile, askopenfilename
from Framing_functions import Create_Frame, Create_text, Create_button, Auto_configure, Create_Heading, Create_entry
import Styles as sty
from functools import partial
import EWOD as pad
import ProgramParameters as param_prog

temp='None'

def get_value_1(Window, entry1, protocole, ProtoWindow): #OK
    """
    This program gets 1 values from a label.
    
    Entry:
        Window(Frame): the protocol window
        entry1(Label variable): the first value to get
        entry2(Label variable): the second value to get
        protocole(list): the list of the protocole
        ProtoWindow(Frame): the protocol frame
    """
    pos=protocole.index(['position'])
    protocole[pos-1]=[protocole[pos-1][0],entry1.get()]
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)
    Window.destroy()

def get_value_temp(Window, entry1,entry2, protocole, ProtoWindow): #OK
    """
    This program gets 2 values from a label.
    
    Entry:
        Window(Frame): the protocol window
        entry1(Label variable): the first value to get
        entry2(Label variable): the second value to get
        protocole(list): the list of the protocole
        ProtoWindow(Frame): the protocol frame
    """
    zone=int(entry1.get())
    temp=int(entry2.get())
    if ((zone!=1) and (zone!=2)):
        messagebox.showerror("Invalid command","You must choose to command the temperature either on zone 1 or on zone 2")
        protocole.remove(['tem'])
    elif (temp>param_prog.temp_max or temp<param_prog.temp_min):
        messagebox.showerror("Invalid command", "You must enter a temperature comprised between 10°C and 100°C")
        protocole.remove(['tem'])
    else:
        pos=protocole.index(['position'])
        protocole[pos-1]=[protocole[pos-1][0],zone, temp]
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)
    Window.destroy()

def get_value_pipette(Window, entry1,entry2, entry3, protocole, ProtoWindow): #OK
    """
    This program gets 2 values from a label.
    
    Entry:
        Window(Frame): the protocol window
        entry1(Label variable): the first value to get
        entry2(Label variable): the second value to get
        protocole(list): the list of the protocole
        ProtoWindow(Frame): the protocol frame
    """
    vol=entry1.get()
    vol=float(vol)
    uni=entry2.get()
    sol=entry3.get()
    pos=protocole.index(['position'])
    if uni=='nL':
        vol=vol/1000
        if vol>param_prog.vol_max:
            messagebox.showinfo(title="Error in volume", message="Volume entered must be comprised between "+str(param_prog.vol_min)+"µL and "+str(param_prog.vol_max)+"µL")
            protocole.remove(['pip'])
    elif vol<param_prog.vol_min:
        messagebox.showinfo(title="Error in volume", message="Volume entered must be comprised between "+str(param_prog.vol_min)+"µL and "+str(param_prog.vol_max)+"µL")
        protocole.remove(['pip'])
    else:
        protocole[pos-1]=[protocole[pos-1][0],str(vol), sol]
    if vol%0.5!=0:
        k=vol//0.5
        if vol%0.5>0.25:
            k=k*0.5
        else:
            k=(k+1)*0.5
        if messagebox.askyesno(title="Error in volume", message="The volume you entered is too precise, do you want to choose the closest allowed volume ("+str(k)+" µL)?"):
            protocole[pos-1]=[protocole[pos-1][0],str(k), sol]
        else:
            protocole.remove(['pip'])
    Window.destroy()
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)

def get_value_measure(Window, entry1,entry2, entry3, protocole, ProtoWindow): #OK
    """
    This program gets 2 values from a label.
    
    Entry:
        Window(Frame): the protocol window
        entry1(Label variable): the first value to get
        entry2(Label variable): the second value to get
        protocole(list): the list of the protocole
        ProtoWindow(Frame): the protocol frame
    """
    nbmes=int(entry1.get())
    tme=float(entry2.get())
    uni=entry3.get()
    pos=protocole.index(['position'])
    if uni=='min':
        if (tme%1)!=0:
            if (messagebox.askyesno("Entry Precision", "If you meant "+str(int((tme*60)//60))+" min, "+str(int((tme*60)%60))+" s, press Yes")):
                tme=int(((tme*60)//60)*60+((tme*60)%60))
            elif(messagebox.askyesno("Entry Precision", "If you meant "+str(int((tme*60)//60))+" min, "+str(int((tme%1)*100))+" s, press Yes")):
                tme=int(((tme*60)//60)*60+(tme%1)*100)
            else:
                protocole.remove(['lum'])
        messagebox.showinfo("Temperature risk","The captor zone is not commanded in temperature. \n If temperature is a factor, please move the drop back to a temperature zone until a new measurement is needed, and move it back under the captor to redo the measurement")
        protocole[pos-1]=[protocole[pos-1][0],str(nbmes),str(tme)]    
    else:
        protocole[pos-1]=[protocole[pos-1][0],str(nbmes),str(tme)]
    if tme<1:
        messagebox.showerror("Not enough time between measurements", "For each measure, an average over a few hundreds of points is taken. To avoid any error, please wait at least 1s between each measurement")
        protocole.remove(['lum'])
    Window.destroy()
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)

def get_value_prot(nb_iterations, protocole, Window_option,ProtoWindow):
    """
    This function gets the values from the choice of iterating a protocol
    Entry:
        file(str): the path of the protocol to iterate
        nb_iterations(entry): the number of times we will iterate the protocol
        protocole(list): the list with all the steps of the protocol
        Window_option(Frame): the window that allows to select the protocol to implement
        ProtoWindow(Frame): the protocol frame
    """
    global temp
    file=temp
    mes=nb_iterations.get()
    pos=protocole.index(['position'])
    protocole[pos-1]=[protocole[pos-1][0],str(file),str(mes)]
    Window_option.destroy()
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)

def get_value_4(Window, entry1, entry2, entry3, entry4, protocole, ProtoWindow): #OK
    """
    This program gets 4 values from a label.
    
    Entry:
        Window(Frame): the protocol window
        entry1(Label variable): the first value to get
        entry2(Label variable): the second value to get
        entry3(Label variable): the third value to get
        entry4(Label variable): the fourth value to get
        protocole(list): the list of the protocole
        ProtoWindow(Frame): the protocol frame
    """
    pos=protocole.index(['position'])
    protocole[pos-1]=[protocole[pos-1][0],entry1.get(), entry2.get(), entry3.get(), entry4.get()]
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)
    Window.destroy()

def get_values_matrix(Window, x, y, temp_file, ProtoWindow): #OK
    """
    This program gets 2 positions from a matrix of labels (it is called 2 times each to add one position).
    
    Entry:
        Window(Frame): the protocol window
        x: the first value to get
        y(Label variable): the second value to get
        temp_file(string): the name of the file
        ProtoWindow(Frame): the protocol frame
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    protocole[pos-1]=protocole[pos-1]+[x,y]
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    if len(protocole[pos-1])==5:
        xbeg=protocole[pos-1][1]
        ybeg=protocole[pos-1][2]
        xend=protocole[pos-1][3]
        yend=protocole[pos-1][4]
        result=messagebox.askyesno("Confirm movement","Is the following movement correct?\n ("+str(xbeg)+","+str(ybeg)+") to ("+str(xend)+","+str(yend)+")")
        if not result:
            protocole=protocole[:pos-1]+protocole[pos:]
        Prot.AfficherProtocol(ProtoWindow,protocole)
        Window.destroy()

def pipet(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    if a==[]:
        a=[['pip']]
    else:
        a=a+[['pip']]
    b=protocole[pos:]
    #Create the new window
    Window_option=Toplevel(Window)
    Window_option.title('Neurodrop - Pipette calibration')
    #Create the heading of the new window
    Heading=Create_Heading(Window_option)   
    #Create the choices of solutions and the slider for the volume
    Solution=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=2, nbcolumns=8, form='wens')
    Create_text(Solution, 'Choose the solution', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=4, nblines=1)
    var=IntVar()
    for i in range(1,param_prog.nbsol+1):
        R = Radiobutton(Solution, text="Solution "+str(i), variable=var, value=i, bg=sty.Bkg_frm_blue)
        R.grid(column=(i-1)%4+1,row=3+(i-1)//4, columnspan=1, rowspan=1, sticky='we')
    #Choice of the volume to pipette
    Volume=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=5, nblines=2, nbcolumns=8, form='wens')
    Create_text(Volume, 'Choose the volume', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=5, nblines=1)
    Vol=Entry(Volume, bd=2)
    Vol.grid(column=1,row=3, columnspan=3,rowspan=1)
    Units=Spinbox(Volume,values=['µL','nL'])
    Units.grid(column=5, row=3, columnspan=1, rowspan=1)
    #Confirm panel
    Confirm=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=8, nblines=1, nbcolumns=8, form='wens')
    Create_text(Confirm, 'Save parameters', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_button(Confirm,'OK', partial(get_value_pipette,Window_option, Vol, Units,var,a+b, ProtoWindow)).grid(column=1,row=3, columnspan=1, rowspan=1, sticky='wens')
    #Configure the window
    Auto_configure(Window_option,lines=[0,2,3,4,6,7,8],columns=[1,2,3,4,5,6,7,8],min_size_col=10,min_size_row=10,weight_col=0,weight_row=0)
    Auto_configure(Window_option,lines=[1,5,9],columns=[0,9],min_size_col=10,min_size_row=10,weight_col=1,weight_row=1)
    Auto_configure(Heading,lines=[0],columns=[0,1,2,3,4,5,6,7],min_size_col=10,min_size_row=10,weight_col=1,weight_row=0)
    Auto_configure(Solution,lines=[0,2,param_prog.nbsol],columns=[0,5],min_size_col=10,min_size_row=10,weight_col=1,weight_row=1)
    Auto_configure(Solution,lines=[1]+[3+i for i in range(param_prog.nbsol)],columns=[1,2,3,4],min_size_col=10,min_size_row=10,weight_col=0,weight_row=0)
    Auto_configure(Volume,lines=[0,2,4],columns=[0,3,6],min_size_col=10,min_size_row=10,weight_col=1,weight_row=1)
    Auto_configure(Volume,lines=[1,3],columns=[1],min_size_col=10,min_size_row=10,weight_col=0,weight_row=0)
    Auto_configure(Confirm,lines=[0,2,4],columns=[0,2],min_size_col=10,min_size_row=10,weight_col=1,weight_row=1)
    Auto_configure(Confirm,lines=[1,3],columns=[1],min_size_col=10,min_size_row=10,weight_col=0,weight_row=0)
    return 1

def move(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    special=[(pad.l_drop,pad.col_drop), (pad.l_capt,pad.col_capt)]
    if a==[]:
        a=[['mov']]
    else:
        a=a+[['mov']]
    b=protocole[pos:]
    Prot.EcrireProtocoleFichier(a+b,'temp.txt')
    #Create the new window
    Window_option=Toplevel(Window)
    Window_option.title('Neurodrop - Movement calibration')
    #Create the heading of the new window
    Heading=Create_Heading(Window_option)   
    #Create the button matrix to choose the displacement

    canvas = Canvas(Window_option,width=1000,height=600)
    scroll_y = Scrollbar(Window_option, orient="vertical")
    canvas.config(yscrollcommand=scroll_y.set)

    fichier=open(temp_file, 'r')
    actual=fichier.readlines()
    fichier.close()
    pad_activ=[]
    end=actual.index('position\n')
    for i in range(end-1):
        if actual[i][0:3]=='mov':
            K=actual[i]
            K=K[:-1]
            K=K.split(',')
            for h in range(1,len(K)):
                K[h]=int(K[h])
            pad_activ=pad_activ+[(K[3],K[4])]
            special=special+[(K[3],K[4])]

    Matrix=Create_Frame(canvas, border_thickness=0, style_border='solid', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=6, nbcolumns=8, form='wens')
    fichier=open('list_pad.csv','r') # We generate the pads track from a csv file (list_pads), which is a data of the program
    lines=fichier.readlines()
    fichier.close()
    for i in range(len(lines)): # From the file we create lines, which is a list of lists containing the value of the pad if there is a pad and 0 if there is nothing
        lines[i]=lines[i].replace('\n',"")
        lines[i]=lines[i].split(';')
    lines[0][0]=lines[0][0][1:]
    for l in range(len(lines)):
        lin=l
        for col in range(len(lines[l])):
            if ((l,col)==(pad.l_capt,pad.col_capt)):
                #Create_button(Matrix, '', partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), activefg=sty.Bkg_button_active, Width=2, Height=1, Police='Arial 10', col=1+col, line=3+l, form='wens', nblines=1, pady=0)
                Label_button=Button(Matrix,text='C', bg="orange", width=2, height=1, font='Arial 15 bold', command=partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), pady=0)
                Label_button.grid(column=1+col,row=5+l,sticky='wens')
            elif((l,col)==(pad.l_drop,pad.col_drop)):
                Label_button=Button(Matrix,text='D', bg="orange", width=2, height=1, font='Arial 15 bold', command=partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), pady=0)
                Label_button.grid(column=1+col,row=5+l,sticky='wens')
            elif(pad.l_beg_z_1<=l and pad.l_end_z_1>=l and pad.col_beg_z_1<=col and pad.col_end_z_1>=col and ((l,col) not in pad_activ)):
                Label_button=Button(Matrix,text='Z1', bg="orange", width=2, height=1, font='Arial 15 bold', command=partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), pady=0)
                Label_button.grid(column=1+col,row=5+l,sticky='wens')
            elif(pad.l_beg_z_2<=l and pad.l_end_z_2>=l and pad.col_beg_z_2<=col and pad.col_end_z_2>=col and ((l,col) not in pad_activ)):
                Label_button=Button(Matrix,text='Z2', bg="orange", width=2, height=1, font='Arial 15 bold', command=partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), pady=0)
                Label_button.grid(column=1+col,row=5+l,sticky='wens')
            elif((l,col) in pad_activ):
                Label_button=Button(Matrix,text='X', bg="orange", width=2, height=1, font='Arial 15 bold', command=partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), pady=0)
                Label_button.grid(column=1+col,row=5+l,sticky='wens')
            elif((lines[l][col]!='0') & ((l,col) not in special)):
                Create_button(Matrix, 'O', partial(get_values_matrix,Window_option,l,col,temp_file, ProtoWindow), activefg=sty.Bkg_button_active, Width=2, Height=1, Police='Arial 15 italic', col=1+col, line=5+l, form='wens', nblines=1, pady=0)
    Create_text(Matrix, 'Select the departure and the arriving pads', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=col+1, nblines=1)
    Label(Matrix,text='LEGEND: \n \t - D: the pad where the solution is droped by the pipette \n \t - C: the pad right under the light sensor \n \t - Z1: the first zone which temperature can be controlled \n \t - Z2: the second zone which temperature can be controlled \n \t - O: pad not occupied with no special properties \n \t - X: pad occupied',foreground=sty.Clr_txt_frames,font=sty.Font_filepath,background=sty.Bkg_frm_blue, justify='left').grid(column=1,row=3, columnspan=col+1, rowspan=1, sticky='wns')
    Create_text(Matrix, 'END', color=sty.Bkg_frm_blue, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=lin+7, form='wens', nbcolumns=col, nblines=1)
    canvas.create_window(0, 0, anchor='nw', window=Matrix)
    canvas.update_idletasks()

    canvas.configure(scrollregion=canvas.bbox('all'), yscrollcommand=scroll_y.set)


    canvas.grid(row=2,column=1,rowspan=6,columnspan=7, sticky='wens')
    scroll_y.grid(row=2,column=8, rowspan=6, columnspan=1, sticky='ns')
    scroll_y.configure(command=canvas.yview)

    #Configure the window
    W=[Window_option, Window_option, Heading, Matrix, Matrix]
    Lines=[[0,2,3,4,5,6,7,8], [1,9], [0], [0,2,4,lin+7], [1,3]+[i for i in range(5,lin+6)]]
    Columns=[[1,2,3,4,5,6,7,8], [0,9], [0,1,2,3,4,5,6,7], [1]+[i for i in range(2,col+2)], [0,col+2]]
    WC=[0,1,1,1,0,1,0]
    WL=[0,1,0,1,0,1,0]
    for i in range(len(W)):
        Auto_configure(W[i],lines=Lines[i],columns=Columns[i],min_size_col=10,min_size_row=10,weight_col=WC[i],weight_row=WL[i])

def measure(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    if a==[]:
        a=[['lum']]
    else:
        a=a+[['lum']]
    b=protocole[pos:]
    #Create the new window
    Window_option=Toplevel(Window)
    Window_option.title('Neurodrop - Measurement calibration')
    #Create the heading of the new window
    Heading=Create_Heading(Window_option)   
    #Create the choices of solutions and the slider for the volume
    Nb_measurements=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=2, nbcolumns=8, form='wens')
    Create_text(Nb_measurements, 'Enter the number of measurements to perform', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    nb_mes=Entry(Nb_measurements, bd=2)
    nb_mes.grid(column=1,row=3, columnspan=2,rowspan=1)


    Time_inter=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=5, nblines=2, nbcolumns=8, form='wens')
    Create_text(Time_inter, 'Enter the time between each measurements', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=3, nblines=1)

    Val=Entry(Time_inter, bd=2)
    Val.grid(column=1,row=3, columnspan=1,rowspan=1)
    Units=Spinbox(Time_inter,values=['s','min'])
    Units.grid(column=2, row=3, columnspan=1, rowspan=1)

    Confirm=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=8, nblines=1, nbcolumns=8, form='wens')
    Create_text(Confirm, 'Save parameters', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_button(Confirm,'OK', partial(get_value_measure,Window_option, nb_mes, Val, Units, a+b, ProtoWindow)).grid(column=1,row=3, columnspan=1, rowspan=1, sticky='wens')
    
    #Configure the window
    W=[Window_option, Window_option, Heading, Nb_measurements, Nb_measurements, Time_inter, Time_inter, Confirm, Confirm]
    Lines=[[0,2,3,4,6,7,8], [1,5,9], [0], [0,2,4], [1,3], [0,2,4], [1,3], [0,2,4], [1,3]]
    Columns=[[1,2,3,4,5,6,7,8], [0,9], [0,1,2,3,4,5,6,7], [0,2], [1], [0,2],[1], [0,2],[1]]
    WC=[0,1,1,1,0,1,0,1,0]
    WL=[0,1,0,1,0,1,0,1,0]
    for i in range(len(W)):
        Auto_configure(W[i],lines=Lines[i],columns=Columns[i],min_size_col=10,min_size_row=10,weight_col=WC[i],weight_row=WL[i])

    return 1

def clean(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    if a==[]:
        a=[['cle']]
    else:
        a=a+[['cle']]
    b=protocole[pos:]
    protocole=a+b
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)
    return 1

def thaw(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    if a==[]:
        a=[['tha']]
    else:
        a=a+[['tha']]
    b=protocole[pos:]
    protocole=a+b
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    Prot.AfficherProtocol(ProtoWindow,protocole)
    return 1

def temperature(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the temperature instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    messagebox.showinfo("Placement of the temperature zones", "The EWOD plate has two zones with commanded temperature: \n \t - Zone 1: At the left \n \t - Zone 2: At the right")
    a=protocole[:pos]
    if a==[]:
        a=[['tem']]
    else:
        a=a+[['tem']]
    b=protocole[pos:]
    #Create the new window
    Window_option=Toplevel(Window)
    Window_option.title('Neurodrop - Temperature calibration')
    #Create the heading of the new window
    Heading=Create_Heading(Window_option)   
    #Create the choices of solutions and the slider for the volume
    Zone=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=2, nbcolumns=8, form='wens')
    Create_text(Zone, 'Select the zone you want to control', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_text(Zone, 'Zone 1 is on the left, Zone 2 on the right', color=sty.Clr_txt_frames, Police=sty.Font_filepath, back_colour=sty.Bkg_frm_blue, col=1, line=2, form='wens', nbcolumns=1, nblines=1)

    zon=StringVar()
    Create_entry(Zone, zon, Width=30, Police='Arial 25', col=1, line=4, form='wens', nblines=1)

    Temp=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=5, nblines=2, nbcolumns=8, form='wens')
    Create_text(Temp, 'Select the temperature of the zone (in degrees C)', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    tem=StringVar()
    Create_entry(Temp, tem, Width=30, Police='Arial 25', col=1, line=3, form='wens', nblines=1)

    Confirm=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=8, nblines=1, nbcolumns=8, form='wens')
    Create_text(Confirm, 'Save parameters', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_button(Confirm,'OK', partial(get_value_temp,Window_option, zon, tem, a+b, ProtoWindow)).grid(column=1,row=3, columnspan=1, rowspan=1, sticky='wens')
    
    #Configure the window
    W=[Window_option, Window_option, Heading, Zone, Zone, Temp, Temp, Confirm, Confirm]
    Lines=[[0,2,3,4,6,7,8], [1,5,9], [0], [0,3,5], [1,4], [0,2,4], [1,3], [0,2,4], [1,3]]
    Columns=[[1,2,3,4,5,6,7,8], [0,9], [0,1,2,3,4,5,6,7], [0,2], [1], [0,2],[1], [0,2],[1]]
    WC=[0,1,1,1,0,1,0,1,0]
    WL=[0,1,0,1,0,1,0,1,0]
    for i in range(len(W)):
        Auto_configure(W[i],lines=Lines[i],columns=Columns[i],min_size_col=10,min_size_row=10,weight_col=WC[i],weight_row=WL[i])

    return 1

def wait(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the temperature instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    if a==[]:
        a=[['wai']]
    else:
        a=a+[['wai']]
    b=protocole[pos:]
    #Create the new window
    Window_option=Toplevel(Window)
    Window_option.title('Neurodrop - Waiting calibration')
    #Create the heading of the new window
    Heading=Create_Heading(Window_option)   
    #Create the choices of solutions and the slider for the volume
    Tme=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=5, nbcolumns=8, form='wens')
    Create_text(Tme, 'Define the number of seconds you want to wait', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    tim=StringVar()
    Create_entry(Tme, tim, Width=30, Police='Arial 25', col=1, line=3, form='wens', nblines=1)

    Confirm=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=8, nblines=1, nbcolumns=8, form='wens')
    Create_text(Confirm, 'Save parameters', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_button(Confirm,'OK', partial(get_value_1,Window_option, tim, a+b, ProtoWindow)).grid(column=1,row=3, columnspan=1, rowspan=1, sticky='wens')
    
    #Configure the window
    W=[Window_option, Window_option, Heading, Tme, Tme, Confirm, Confirm]
    Lines=[[0,2,3,4,6,7,8], [1,5,9], [0], [0,2,4], [1,3], [0,2,4], [1,3]]
    Columns=[[1,2,3,4,5,6,7,8], [0,9], [0,1,2,3,4,5,6,7], [0,2], [1], [0,2],[1]]
    WC=[0,1,1,1,0,1,0,1,0]
    WL=[0,1,0,1,0,1,0,1,0]
    for i in range(len(W)):
        Auto_configure(W[i],lines=Lines[i],columns=Columns[i],min_size_col=10,min_size_row=10,weight_col=WC[i],weight_row=WL[i])

    return 1

def PosUp(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    if pos==0:
        messagebox.showerror("Operation not allowed","You have reached the first position in the protocol")
    else:
        sze=len(protocole)
        a=protocole[:pos-1]
        b=[protocole[pos-1]]
        if pos<sze:
            c=protocole[pos+1:]
        else:
            c=[]
        protocole=a+[['position']]+b+c
        Prot.EcrireProtocoleFichier(protocole,'temp.txt')
        Prot.AfficherProtocol(ProtoWindow,protocole)
    return 1

def PosDown(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    sze=len(protocole)
    if(pos+1==sze):
        messagebox.showerror("No more positions below", "You have reached the last position in the protocol")
    else:
        a=protocole[:pos]
        if pos<sze:
            b=[protocole[pos+1]]
            c=protocole[pos+2:]
        else:
            b=[]
            c=[]
        protocole=a+b+[['position']]+c
        Prot.EcrireProtocoleFichier(protocole,'temp.txt')
        Prot.AfficherProtocol(ProtoWindow,protocole)
    return 1

def Save(Window, ProtoWindow, temp_file): #OK
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    os.remove(temp_file)
    pos=protocole.index(['position'])
    protocole=protocole[:pos]+protocole[pos+1:]
    filename=asksaveasfile(title="Save the protocol as",filetypes=[('txt files','.txt'),('all files','.*')])
    filename=str(filename)
    filename=filename.split("'")
    filename=filename[1]
    Prot.EcrireProtocoleFichier(protocole,filename)
    Window.destroy()
    return 1

def Delete(Window, ProtoWindow, temp_file): #Still a problem to remove the last element
    """
    This program adds the pipeting instruction.
    
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    os.remove(temp_file)
    pos=protocole.index(['position'])
    if pos>1:
        a=protocole[:pos-1]
    else:
        a=[]
    b=protocole[pos:]
    protocole=a+b
    Prot.EcrireProtocoleFichier(protocole,'temp.txt')
    ProtoWindow=Prot.AfficherProtocol(ProtoWindow,protocole)
    return 1

def other_protocol(Window_protocol, ProtoWindow, temp_file):
    """
    This function asks the user to choose an existing protocol and how many times he wants to apply it in the main protocol
    Entry:
        Window(Frame): the protocol window
        ProtoWindow(Frame): the protocol frame
        temp_file(string): the name of the file
    """
    protocole=Prot.ReadProtocolFile(temp_file)
    pos=protocole.index(['position'])
    a=protocole[:pos]
    if a==[]:
        a=[['pro']]
    else:
        a=a+[['pro']]
    b=protocole[pos:]
    protocole=a+b
    #Create the new window
    Window_option=Toplevel(ProtoWindow)
    Window_option.title('Neurodrop - Apply a sub-protocol')
    #Create the heading of the new window
    Heading=Create_Heading(Window_option)  
    #Choose the protocol:
    Protocol=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=2, nblines=2, nbcolumns=6, form='wens')
    Create_text(Protocol, 'Select the file of the sub-protocol', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_button(Protocol,'Select', partial(choose_sub_protocol, Protocol)).grid(column=1,row=3, columnspan=1, rowspan=1, sticky='wens')

    #Ask for the number of iterations of the protocol
    Nb_measurements=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=5, nblines=2, nbcolumns=6, form='wens')
    Create_text(Nb_measurements, 'Enter the number of iteration of the sub-protocol', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    nb_mes=Entry(Nb_measurements, bd=2)
    nb_mes.grid(column=1,row=3,columnspan=2,rowspan=1)
    
    #Confirm choice
    Confirm=Create_Frame(Window_option, border_thickness=5, style_border='groove', back_colour=sty.Bkg_frm_blue, col=1, line=8, nblines=1, nbcolumns=6, form='wens')
    Create_text(Confirm, 'Confirm choice', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    Create_button(Confirm, 'OK', partial(get_value_prot,nb_mes, protocole, Window_option,ProtoWindow)).grid(column=1, row=3,columnspan=1,rowspan=1,sticky='wens')

    Auto_configure(Heading, lines=[0], columns=[0,1,2,3,4,5,6,7], min_size_row=10, min_size_col=30, weight_col=1, weight_row=0)
    Auto_configure(Window_option, lines=[0,2,3,5,6,8], columns=[1,2,3,4,5,6], min_size_row=10, min_size_col=30, weight_col=1, weight_row=1)
    Auto_configure(Window_option, lines=[1,4,7,9], columns=[0,7], min_size_row=30, min_size_col=10, weight_col=0, weight_row=0)
    Auto_configure(Protocol, lines=[1,3], columns=[1], min_size_row=10, min_size_col=30, weight_col=0, weight_row=0)
    Auto_configure(Protocol, lines=[0,2,4], columns=[0,2], min_size_row=10, min_size_col=30, weight_col=0, weight_row=0)
    Auto_configure(ProtoWindow, lines=[0,2,3,5,6,8], columns=[1], min_size_row=10, min_size_col=30, weight_col=1, weight_row=1)
    Auto_configure(ProtoWindow, lines=[1,4,7], columns=[0,2], min_size_row=30, min_size_col=30, weight_col=1, weight_row=1)
    Auto_configure(Nb_measurements, lines=[0,2,4], columns=[0,2], min_size_row=10, min_size_col=30, weight_col=0, weight_row=0)
    Auto_configure(Nb_measurements, lines=[1,3], columns=[1], min_size_row=30, min_size_col=30, weight_col=1, weight_row=1)
    Auto_configure(Confirm, lines=[0,2,4], columns=[0,2], min_size_row=10, min_size_col=30, weight_col=0, weight_row=0)
    Auto_configure(Confirm, lines=[1,3], columns=[1], min_size_row=30, min_size_col=30, weight_col=1, weight_row=1)

def choose_sub_protocol(Protocol_Window):
    """
    This function asks for the user to choose a sub-protocol, then changes the previous window to show the protocol
    Entry: 
        Protocol_Window: the Window where we will show the protocol
        temp: the list that will get the filepath
    """
    filepath = askopenfilename(title="Open a protocol",filetypes=[('txt files','.txt'),('all files','.*')])
    global temp
    temp = filepath
    for widget in Protocol_Window.winfo_children():
        widget.destroy()
    protocol=Prot.ReadProtocolFile(filepath)
    Create_text(Protocol_Window, 'Protocol selected:', color=sty.Clr_txt_frames, Police=sty.Font_frames_title, back_colour=sty.Bkg_frm_blue, col=1, line=1, form='wens', nbcolumns=1, nblines=1)
    scroll_prot=Scrollbar(Protocol_Window)
    scroll_prot.grid(column=2, row=3, rowspan=1, columnspan=1, sticky='ns')
    listbox = Listbox(Protocol_Window, yscrollcommand=scroll_prot.set)
    for step in protocol:
        cmd=step[0]
        if cmd=='pip':
            vol=step[1]
            sol=step[2]
            listbox.insert('end','Pipette '+str(vol)+' µL in solution '+str(sol))
        elif cmd=='dep':
            listbox.insert('end','Leave the pipetted solution on the EWOD plate')
        elif cmd=='mov':
            xbeg=step[1]
            ybeg=step[2]
            xend=step[3]
            yend=step[4]
            listbox.insert('end','Move the droplet from ('+str(xbeg)+','+str(ybeg)+') to ('+str(xend)+','+str(yend)+')')
        elif cmd=='lum':
            nbmes=step[1]
            t_inter_mes=step[2]
            listbox.insert('end','Do '+str(nbmes)+' measures of luminescence, waiting '+str(t_inter_mes)+' s between each measure')
        elif cmd=='cle':
            listbox.insert('end','Clean the EWOD plate')
        elif cmd=='tha':
            listbox.insert('end','Thaw the solutions')
        elif cmd=='tem':
            zone=step[1]
            temperature=step[2]
            listbox.insert('end','Put zone '+str(zone)+' to '+str(temperature)+' degrees C')
        elif cmd=='wai':
            tim=step[1]
            listbox.insert('end','Wait '+str(tim)+'s')
        elif cmd=='pro':
            path=step[1]
            temp=path.split('/')
            listbox.insert('end','Apply the sub-protocol: '+str(temp[-1]))
    listbox.grid(column=1, row=3, rowspan=1, columnspan=1, sticky='wens')
    scroll_prot.config(command=listbox.yview)
    Create_button(Protocol_Window,'Change', partial(choose_sub_protocol, Protocol_Window)).grid(column=1,row=5, columnspan=1, rowspan=1, sticky='wens')
    Auto_configure(Protocol_Window,lines=[0,2,4,6],columns=[0,2])
    Auto_configure(Protocol_Window,lines=[1,3,5],columns=[1],min_size_row=5,weight_col=0,weight_row=0)
    return temp[0]


